/*
 * Copyright 2005-2010 NVIDIA Corporation.  All rights reserved.
 */

/*

  This program is free software; you can redistribute it and/or modify it
  under the terms of version 2 of the GNU General Public License as
  published by the Free Software Foundation.

  This program is distributed in the hope that it would be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

  Further, this software is distributed without any warranty that it is
  free of the rightful claim of any third person regarding infringement 
  or the like.  Any license provided herein, whether implied or 
  otherwise, applies only to this software file.  Patent licenses, if 
  any, provided herein do not apply to combinations of this program with 
  other software, or any other product whatsoever.  

  You should have received a copy of the GNU General Public License along
  with this program; if not, write the Free Software Foundation, Inc., 59
  Temple Place - Suite 330, Boston MA 02111-1307, USA.

*/


//  
//  Generate ABI information
///////////////////////////////////////


//  $Revision: 1.22 $
//  $Date: 2001/03/10 01:15:59 $
//  $Author: mtibuild $
//  $Source: /osprey.src/osprey1.0/common/targ_info/abi/ia64/RCS/abi_properties.cxx,v $

#include <stddef.h>
#include "abi_properties_gen.h"
#include "targ_isa_registers.h"

static ABI_PROPERTY
	allocatable,
	func_arg,	// params
	func_val,	// retval
	unused,		// unused dummy result when only using cc
        // Define special regs here; these are really part of isa, not abi, 
        // but here is where we have ability to assign a dedicated reg to them.
        // tid/ctaid are 16bit in ptx1, 32bit in ptx2,
        // so define two variants which cg will have to choose between.
	tid1,		// thread id
	ntid1,		// number of thread ids
	ctaid1,		// cta id
	nctaid1,	// number of cta ids
	tid,		// thread id
	ntid,		// number of thread ids
	ctaid,		// cta id
	nctaid,		// number of cta ids
        laneid,         // lane id
        warpid,         // warp id
        nwarpid,        // number of warp ids
        smid,           // sm id
        nsmid,          // number of sm ids
        gridid,         // grid id
        lanemask,       // mask with bit set in position relative to lane number
        pm,             // perf monitor counter
	clock,		// cycle counter
        clock64;        // cycle counter 64b

#define ISA_REGISTER_CLASS_Last_Reg(x) \
	ISA_REGISTER_CLASS_INFO_Last_Reg(ISA_REGISTER_CLASS_Info(x))

#define MAX_PARAM_REGS 96 
#define MAX_RETURN_REGS 36

static const char *i32param_names[MAX_PARAM_REGS] = {
  "%ra1", "%ra2", "%ra3", "%ra4", "%ra5", "%ra6", "%ra7", "%ra8", 
  "%ra9", "%ra10", "%ra11", "%ra12", "%ra13", "%ra14", "%ra15", "%ra16",
  "%ra17", "%ra18", "%ra19", "%ra20", "%ra21", "%ra22", "%ra23", "%ra24",
  "%ra25", "%ra26", "%ra27", "%ra28", "%ra29", "%ra30", "%ra31", "%ra32",
  "%ra33", "%ra34", "%ra35", "%ra36", "%ra37", "%ra38", "%ra39", "%ra40",
  "%ra41", "%ra42", "%ra43", "%ra44", "%ra45", "%ra46", "%ra47", "%ra48", 
  "%ra49", "%ra50", "%ra51", "%ra52", "%ra53", "%ra54", "%ra55", "%ra56", 
  "%ra57", "%ra58", "%ra59", "%ra60", "%ra61", "%ra62", "%ra63", "%ra64",
  "%ra65", "%ra66", "%ra67", "%ra68", "%ra69", "%ra70", "%ra71", "%ra72",
  "%ra73", "%ra74", "%ra75", "%ra76", "%ra77", "%ra78", "%ra79", "%ra80",
  "%ra81", "%ra82", "%ra83", "%ra84", "%ra85", "%ra86", "%ra87", "%ra88",
  "%ra89", "%ra90", "%ra91", "%ra92", "%ra93", "%ra94", "%ra95", "%ra96"};
static const char *i32ret_names[MAX_RETURN_REGS] = {
  "%rv1", "%rv2", "%rv3", "%rv4", "%rv5", "%rv6", "%rv7", "%rv8", 
  "%rv9", "%rv10", "%rv11", "%rv12", "%rv13", "%rv14", "%rv15", "%rv16",
  "%rv17", "%rv18", "%rv19", "%rv20", "%rv21", "%rv22", "%rv23", "%rv24",
  "%rv25", "%rv26", "%rv27", "%rv28", "%rv29", "%rv30", "%rv31", "%rv32",
  "%rv33", "%rv34", "%rv35", "%rv36"};
static const char *i64param_names[MAX_PARAM_REGS] = {
  "%rda1", "%rda2", "%rda3", "%rda4", "%rda5", "%rda6", "%rda7", "%rda8", 
  "%rda9", "%rda10", "%rda11", "%rda12", "%rda13", "%rda14", "%rda15", "%rda16",
  "%rda17", "%rda18", "%rda19", "%rda20", "%rda21", "%rda22", "%rda23", "%rda24",
  "%rda25", "%rda26", "%rda27", "%rda28", "%rda29", "%rda30", "%rda31", "%rda32",
  "%rda33", "%rda34", "%rda35", "%rda36", "%rda37", "%rda38", "%rda39", "%rda40",
  "%rda41", "%rda42", "%rda43", "%rda44", "%rda45", "%rda46", "%rda47", "%rda48", 
  "%rda49", "%rda50", "%rda51", "%rda52", "%rda53", "%rda54", "%rda55", "%rda56", 
  "%rda57", "%rda58", "%rda59", "%rda60", "%rda61", "%rda62", "%rda63", "%rda64",
  "%rda65", "%rda66", "%rda67", "%rda68", "%rda69", "%rda70", "%rda71", "%rda72",
  "%rda73", "%rda74", "%rda75", "%rda76", "%rda77", "%rda78", "%rda79", "%rda80",
  "%rda81", "%rda82", "%rda83", "%rda84", "%rda85", "%rda86", "%rda87", "%rda88",
  "%rda89", "%rda90", "%rda91", "%rda92", "%rda93", "%rda94", "%rda95", "%rda96"};
static const char *i64ret_names[MAX_RETURN_REGS] = {
  "%rdv1", "%rdv2", "%rdv3", "%rdv4", "%rdv5", "%rdv6", "%rdv7", "%rdv8", 
  "%rdv9", "%rdv10", "%rdv11", "%rdv12", "%rdv13", "%rdv14", "%rdv15", "%rdv16",
  "%rdv17", "%rdv18", "%rdv19", "%rdv20", "%rdv21", "%rdv22", "%rdv23", "%rdv24",
  "%rdv25", "%rdv26", "%rdv27", "%rdv28", "%rdv29", "%rdv30", "%rdv31", "%rdv32",
  "%rdv33", "%rdv34", "%rdv35", "%rdv36"};
static const char *f32param_names[MAX_PARAM_REGS] = {
  "%fa1", "%fa2", "%fa3", "%fa4", "%fa5", "%fa6", "%fa7", "%fa8", 
  "%fa9", "%fa10", "%fa11", "%fa12", "%fa13", "%fa14", "%fa15", "%fa16",
  "%fa17", "%fa18", "%fa19", "%fa20", "%fa21", "%fa22", "%fa23", "%fa24",
  "%fa25", "%fa26", "%fa27", "%fa28", "%fa29", "%fa30", "%fa31", "%fa32",
  "%fa33", "%fa34", "%fa35", "%fa36", "%fa37", "%fa38", "%fa39", "%fa40",
  "%fa41", "%fa42", "%fa43", "%fa44", "%fa45", "%fa46", "%fa47", "%fa48", 
  "%fa49", "%fa50", "%fa51", "%fa52", "%fa53", "%fa54", "%fa55", "%fa56", 
  "%fa57", "%fa58", "%fa59", "%fa60", "%fa61", "%fa62", "%fa63", "%fa64",
  "%fa65", "%fa66", "%fa67", "%fa68", "%fa69", "%fa70", "%fa71", "%fa72",
  "%fa73", "%fa74", "%fa75", "%fa76", "%fa77", "%fa78", "%fa79", "%fa80",
  "%fa81", "%fa82", "%fa83", "%fa84", "%fa85", "%fa86", "%fa87", "%fa88",
  "%fa89", "%fa90", "%fa91", "%fa92", "%fa93", "%fa94", "%fa95", "%fa96"};
static const char *f32ret_names[MAX_RETURN_REGS] = {
  "%fv1", "%fv2", "%fv3", "%fv4", "%fv5", "%fv6", "%fv7", "%fv8", 
  "%fv9", "%fv10", "%fv11", "%fv12", "%fv13", "%fv14", "%fv15", "%fv16",
  "%fv17", "%fv18", "%fv19", "%fv20", "%fv21", "%fv22", "%fv23", "%fv24",
  "%fv25", "%fv26", "%fv27", "%fv28", "%fv29", "%fv30", "%fv31", "%fv32",
  "%fv33", "%fv34", "%fv35", "%fv36"};
static const char *f64param_names[MAX_PARAM_REGS] = {
  "%fda1", "%fda2", "%fda3", "%fda4", "%fda5", "%fda6", "%fda7", "%fda8", 
  "%fda9", "%fda10", "%fda11", "%fda12", "%fda13", "%fda14", "%fda15", "%fda16",
  "%fda17", "%fda18", "%fda19", "%fda20", "%fda21", "%fda22", "%fda23", "%fda24",
  "%fda25", "%fda26", "%fda27", "%fda28", "%fda29", "%fda30", "%fda31", "%fda32",
  "%fda33", "%fda34", "%fda35", "%fda36", "%fda37", "%fda38", "%fda39", "%fda40",
  "%fda41", "%fda42", "%fda43", "%fda44", "%fda45", "%fda46", "%fda47", "%fda48", 
  "%fda49", "%fda50", "%fda51", "%fda52", "%fda53", "%fda54", "%fda55", "%fda56", 
  "%fda57", "%fda58", "%fda59", "%fda60", "%fda61", "%fda62", "%fda63", "%fda64",
  "%fda65", "%fda66", "%fda67", "%fda68", "%fda69", "%fda70", "%fda71", "%fda72",
  "%fda73", "%fda74", "%fda75", "%fda76", "%fda77", "%fda78", "%fda79", "%fda80",
  "%fda81", "%fda82", "%fda83", "%fda84", "%fda85", "%fda86", "%fda87", "%fda88",
  "%fda89", "%fda90", "%fda91", "%fda92", "%fda93", "%fda94", "%fda95", "%fda96"};
static const char *f64ret_names[MAX_RETURN_REGS] = {
  "%fdv1", "%fdv2", "%fdv3", "%fdv4", "%fdv5", "%fdv6", "%fdv7", "%fdv8", 
  "%fdv9", "%fdv10", "%fdv11", "%fdv12", "%fdv13", "%fdv14", "%fdv15", "%fdv16",
  "%fdv17", "%fdv18", "%fdv19", "%fdv20", "%fdv21", "%fdv22", "%fdv23", "%fdv24",
  "%fdv25", "%fdv26", "%fdv27", "%fdv28", "%fdv29", "%fdv30", "%fdv31", "%fdv32",
  "%fdv33", "%fdv34", "%fdv35", "%fdv36"};

int main()
{
  ABI_Properties_Begin("nvisa");

  allocatable = Create_Reg_Property("allocatable");
  func_arg = Create_Reg_Property("func_arg");
  func_val = Create_Reg_Property("func_val");
  unused = Create_Reg_Property("unused");
  tid1 = Create_Reg_Property("tid1");
  ntid1 = Create_Reg_Property("ntid1");
  ctaid1 = Create_Reg_Property("ctaid1");
  nctaid1 = Create_Reg_Property("nctaid1");
  tid = Create_Reg_Property("tid");
  ntid = Create_Reg_Property("ntid");
  ctaid = Create_Reg_Property("ctaid");
  nctaid = Create_Reg_Property("nctaid");
  laneid = Create_Reg_Property("laneid");
  warpid = Create_Reg_Property("warpid");
  nwarpid = Create_Reg_Property("nwarpid");
  smid = Create_Reg_Property("smid");
  nsmid = Create_Reg_Property("nsmid");
  gridid = Create_Reg_Property("gridid");
  lanemask = Create_Reg_Property("lanemask");
  pm = Create_Reg_Property("pm");
  clock = Create_Reg_Property("clock");
  clock64 = Create_Reg_Property("clock64");

#if 0 // initially, nvisa has no calling convention
  callee = Create_Reg_Property("callee");
  caller = Create_Reg_Property("caller");
#endif 
#if 0 // initially, nvisa has no stack
  frame_ptr = Create_Reg_Property("frame_ptr");
  stack_ptr = Create_Reg_Property("stack_ptr");
#endif

  ///////////////////////////////////////
  Begin_ABI("nvisa");

  // Would be simpler to put these special regs at front,
  // but more user-friendly to put at end so user sees r1,r2,etc
  INT tid1_index = ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_integer16) 
	- 2; // includes last index
  INT ntid1_index = tid1_index - 3;
  INT ctaid1_index = ntid1_index - 3;
  INT nctaid1_index = ctaid1_index - 3;

  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, tid1_index, "%tid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, tid1_index+1, "%tid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, tid1_index+2, "%tid.z");
  Reg_Property_Range (tid1, ISA_REGISTER_CLASS_integer16, 
	tid1_index, tid1_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ntid1_index, "%ntid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ntid1_index+1, "%ntid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ntid1_index+2, "%ntid.z");
  Reg_Property_Range (ntid1, ISA_REGISTER_CLASS_integer16, 
	ntid1_index, ntid1_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ctaid1_index, "%ctaid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ctaid1_index+1, "%ctaid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, ctaid1_index+2, "%ctaid.z");
  Reg_Property_Range (ctaid1, ISA_REGISTER_CLASS_integer16, 
	ctaid1_index, ctaid1_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, nctaid1_index, "%nctaid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, nctaid1_index+1, "%nctaid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, nctaid1_index+2, "%nctaid.z");
  Reg_Property_Range (nctaid1, ISA_REGISTER_CLASS_integer16, 
	nctaid1_index, nctaid1_index+2);

  INT tid_index = ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_integer) 
	- 2; // includes last index
  INT ntid_index = tid_index - 3;
  INT ctaid_index = ntid_index - 3;
  INT nctaid_index = ctaid_index - 3;
  INT laneid_index = nctaid_index - 1;
  INT warpid_index = laneid_index - 1;
  INT nwarpid_index = warpid_index - 1;
  INT smid_index = nwarpid_index - 1;
  INT nsmid_index = smid_index - 1;
  INT gridid_index = nsmid_index - 1;
  INT lanemask_index = gridid_index - 5;
  INT pm_index = lanemask_index - 4;
  INT clock_index = pm_index - 1;
  INT i32param_index = clock_index - MAX_PARAM_REGS;
  INT i32ret_index = i32param_index - MAX_RETURN_REGS;

  Set_Reg_Name(ISA_REGISTER_CLASS_integer, tid_index, "%tid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, tid_index+1, "%tid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, tid_index+2, "%tid.z");
  Reg_Property_Range (tid, ISA_REGISTER_CLASS_integer, 
	tid_index, tid_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ntid_index, "%ntid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ntid_index+1, "%ntid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ntid_index+2, "%ntid.z");
  Reg_Property_Range (ntid, ISA_REGISTER_CLASS_integer, 
	ntid_index, ntid_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ctaid_index, "%ctaid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ctaid_index+1, "%ctaid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, ctaid_index+2, "%ctaid.z");
  Reg_Property_Range (ctaid, ISA_REGISTER_CLASS_integer, 
	ctaid_index, ctaid_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, nctaid_index, "%nctaid.x");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, nctaid_index+1, "%nctaid.y");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, nctaid_index+2, "%nctaid.z");
  Reg_Property_Range (nctaid, ISA_REGISTER_CLASS_integer, 
	nctaid_index, nctaid_index+2);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, laneid_index, "%laneid");
  Reg_Property (laneid, ISA_REGISTER_CLASS_integer, laneid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, warpid_index, "%warpid");
  Reg_Property (warpid, ISA_REGISTER_CLASS_integer, warpid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, nwarpid_index, "%nwarpid");
  Reg_Property (nwarpid, ISA_REGISTER_CLASS_integer, nwarpid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, smid_index, "%smid");
  Reg_Property (smid, ISA_REGISTER_CLASS_integer, smid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, nsmid_index, "%nsmid");
  Reg_Property (nsmid, ISA_REGISTER_CLASS_integer, nsmid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, gridid_index, "%gridid");
  Reg_Property (gridid, ISA_REGISTER_CLASS_integer, gridid_index, -1);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, lanemask_index, "%lanemask_eq");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, lanemask_index+1, "%lanemask_le");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, lanemask_index+2, "%lanemask_lt");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, lanemask_index+3, "%lanemask_ge");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, lanemask_index+4, "%lanemask_gt");
  Reg_Property_Range (lanemask, ISA_REGISTER_CLASS_integer, 
	lanemask_index, lanemask_index+4);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, pm_index, "%pm0");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, pm_index+1, "%pm1");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, pm_index+2, "%pm2");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, pm_index+3, "%pm3");
  Reg_Property_Range (pm, ISA_REGISTER_CLASS_integer, 
	pm_index, pm_index+3);
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, clock_index, "%clock");
  Reg_Property (clock, ISA_REGISTER_CLASS_integer, clock_index, -1);
  Reg_Names(ISA_REGISTER_CLASS_integer, i32param_index, 
	i32param_index+MAX_PARAM_REGS-1, i32param_names);
  Reg_Property_Range (func_arg, ISA_REGISTER_CLASS_integer, 
	i32param_index, i32param_index+MAX_PARAM_REGS-1);
  Reg_Names(ISA_REGISTER_CLASS_integer, i32ret_index, 
	i32ret_index+MAX_RETURN_REGS-1, i32ret_names);
  Reg_Property_Range (func_val, ISA_REGISTER_CLASS_integer, 
	i32ret_index, i32ret_index+MAX_RETURN_REGS-1);

  INT clock64_index = ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_integer64);
  INT i64param_index = clock64_index - MAX_PARAM_REGS;


  Set_Reg_Name(ISA_REGISTER_CLASS_integer64, clock64_index, "%clock64");
  Reg_Property (clock64, ISA_REGISTER_CLASS_integer64, clock64_index, -1);

  INT i64ret_index = i64param_index - MAX_RETURN_REGS;
  Reg_Names(ISA_REGISTER_CLASS_integer64, i64param_index, 
	i64param_index+MAX_PARAM_REGS-1, i64param_names);
  Reg_Property_Range (func_arg, ISA_REGISTER_CLASS_integer64, 
	i64param_index, i64param_index+MAX_PARAM_REGS-1);
  Reg_Names(ISA_REGISTER_CLASS_integer64, i64ret_index, 
	i64ret_index+MAX_RETURN_REGS-1, i64ret_names);
  Reg_Property_Range (func_val, ISA_REGISTER_CLASS_integer64, 
	i64ret_index, i64ret_index+MAX_RETURN_REGS-1);

  INT f32param_index = 
    ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_float) - MAX_PARAM_REGS;
  INT f32ret_index = f32param_index - MAX_RETURN_REGS;
  Reg_Names(ISA_REGISTER_CLASS_float, f32param_index, 
	f32param_index+MAX_PARAM_REGS-1, f32param_names);
  Reg_Property_Range (func_arg, ISA_REGISTER_CLASS_float, 
	f32param_index, f32param_index+MAX_PARAM_REGS-1);
  Reg_Names(ISA_REGISTER_CLASS_float, f32ret_index, 
	f32ret_index+MAX_RETURN_REGS-1, f32ret_names);
  Reg_Property_Range (func_val, ISA_REGISTER_CLASS_float, 
	f32ret_index, f32ret_index+MAX_RETURN_REGS-1);

  INT f64param_index = 
    ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_float64) - MAX_PARAM_REGS;
  INT f64ret_index = f64param_index - MAX_RETURN_REGS;
  Reg_Names(ISA_REGISTER_CLASS_float64, f64param_index, 
	f64param_index+MAX_PARAM_REGS-1, f64param_names);
  Reg_Property_Range (func_arg, ISA_REGISTER_CLASS_float64, 
	f64param_index, f64param_index+MAX_PARAM_REGS-1);
  Reg_Names(ISA_REGISTER_CLASS_float64, f64ret_index, 
	f64ret_index+MAX_RETURN_REGS-1, f64ret_names);
  Reg_Property_Range (func_val, ISA_REGISTER_CLASS_float64, 
	f64ret_index, f64ret_index+MAX_RETURN_REGS-1);

  // for now, no abi so don't need anything defined
  // except allocatable
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_integer16, 1,
	nctaid1_index - 1); // ends before nctaid1 
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_integer, 1,
	i32ret_index - 1); // ends before retval 
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_integer64, 1,
	i64ret_index - 1);
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_float, 1,
	f32ret_index - 1);
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_float64, 1,
	f64ret_index - 1);
  Reg_Property_Range (allocatable, ISA_REGISTER_CLASS_predicate, 1, 
	ISA_REGISTER_CLASS_Last_Reg(ISA_REGISTER_CLASS_predicate));

  Set_Reg_Name(ISA_REGISTER_CLASS_integer16, 0, "_");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer, 0, "_");
  Set_Reg_Name(ISA_REGISTER_CLASS_integer64, 0, "_");
  Set_Reg_Name(ISA_REGISTER_CLASS_float, 0, "_");
  Set_Reg_Name(ISA_REGISTER_CLASS_float64, 0, "_");
  Reg_Property(unused, ISA_REGISTER_CLASS_integer, 0, -1);
  Reg_Property(unused, ISA_REGISTER_CLASS_integer16, 0, -1);
  Reg_Property(unused, ISA_REGISTER_CLASS_integer64, 0, -1);
  Reg_Property(unused, ISA_REGISTER_CLASS_float, 0, -1);
  Reg_Property(unused, ISA_REGISTER_CLASS_float64, 0, -1);

  ABI_Properties_End();

  return 0;
}
